"""An incredibly simple HTTP basic auth implementation."""


from base64 import b64decode, b64encode
from urllib import quote, unquote


class DecodeError(Exception):
    pass


def encode(username, password):
    """Returns an HTTP basic authentication encrypted string given a valid
    username and password.
    """
    return 'Basic ' + b64encode('%s:%s' % (quote(username), quote(password)))


def decode(encoded_str):
    """Decode an encrypted HTTP basic authentication string. Returns a tuple of
    the form (username, password), and raises a DecodeError exception if
    nothing could be decoded.
    """
    split = encoded_str.strip().split(' ')

    # If split is only one element, try to decode the username and password
    # directly.
    if len(split) == 1:
        try:
            username, password = b64decode(split[0]).split(':')
        except:
            raise DecodeError

    # If there are only two elements, check the first and ensure it says
    # 'basic' so that we know we're about to decode the right thing. If not,
    # bail out.
    elif len(split) == 2:
        if split[0].strip().lower() == 'basic':
            try:
                username, password = b64decode(split[1]).split(':')
            except:
                raise DecodeError
        else:
            raise DecodeError

    # If there are more than 2 elements, something crazy must be happening.
    # Bail.
    else:
        raise DecodeError

    return unquote(username), unquote(password)



from unittest import TestCase

from basicauth import decode, DecodeError, encode


class Encode(TestCase):

    def test_prepends_basic_auth(self):
        self.assertTrue(encode('', '').lower().startswith('basic'))

    def test_adds_space_after_basic(self):
        self.assertTrue(encode('', '').lower().startswith('basic '))

    def test_encodes_short_username(self):
        self.assertTrue(encode('', 'password'))

    def test_encodes_short_password(self):
        self.assertTrue(encode('username', ''))

    def test_encodes_long_username(self):
        self.assertTrue(encode('username'*1000000, ''))

    def test_encodes_long_password(self):
        self.assertTrue(encode('', 'password'*1000000))


class Decode(TestCase):

    def test_decodes_empty_username(self):
        self.assertEqual('', decode(encode('', 'password'))[0])

    def test_decodes_empty_password(self):
        self.assertEqual('', decode(encode('username', ''))[1])

    def test_decodes_hashes_only(self):
        username, password = 'username', 'omgawesome!'
        encoded_str = encode(username, password)
        encoded_hash = encoded_str.split(' ')[1]
        self.assertEqual((username, password), decode(encoded_hash))

    def test_decodes_fully_encoded_strings(self):
        username, password = 'username', 'password'
        encoded_str = encode(username, password)
        self.assertEqual((username, password), decode(encoded_str))

    def test_doesnt_decode_invalid_auth_types(self):
        encoded_str = 'error woot'
        self.assertRaises(DecodeError, decode, encoded_str)

    def test_doesnt_decode_invalid_hashes(self):
        encoded_str = 'basic omg hacks what'
        self.assertRaises(DecodeError, decode, encoded_str)

        encoded_str = 'basic omg hacks'
        self.assertRaises(DecodeError, decode, encoded_str)

    def test_properly_escapes_colons(self):
        username, password = 'user:name:', 'pass:word:'
        encoded_str = encode(username, password)
        self.assertEqual((username, password), decode(encoded_str))
